import { Body, Controller, Get, Query, Put, Delete, Post } from '@nestjs/common';
import { ApiOperation, ApiTags } from '@nestjs/swagger';
import { ApiSecurityAuth } from '~/common/decorators/swagger.decorator';
import { ApiResult } from '~/common/decorators/api-result.decorator';
import { IdParam } from '~/common/decorators/id-param.decorator';
import { Perm, definePermission } from '~/modules/auth/decorators/permission.decorator';
import { SaleQuotationComponentDto, SaleQuotationComponentQueryDto, SaleQuotationComponentUpdateDto } from './sale_quotation_component.dto';
import { SaleQuotationComponentService } from './sale_quotation_component.service';
export const permissions = definePermission('sale_quotation:sale_quotation_component', {
  LIST: 'list',
  CREATE: 'create',
  READ: 'read',
  UPDATE: 'update',
  DELETE: 'delete'
} as const);

@ApiTags('SaleQuotationComponent - 报价配件')
@ApiSecurityAuth()
@Controller('sale_quotation_component')
export class SaleQuotationComponentController {
  constructor(private saleQuotationComponentService: SaleQuotationComponentService) {}

  @Get()
  @ApiOperation({ summary: '分页获取报价配件列表' })
  @ApiResult({ type: [SaleQuotationComponentDto], isPage: true })
  @Perm(permissions.LIST)
  async list(@Query() dto: SaleQuotationComponentQueryDto) {
    return this.saleQuotationComponentService.findAll(dto);
  }

  @Get(':id')
  @ApiOperation({ summary: '获取报价配件信息' })
  @ApiResult({ type: SaleQuotationComponentDto })
  @Perm(permissions.READ)
  async info(@IdParam() id: number) {
    return this.saleQuotationComponentService.info(id);
  }

  @Post()
  @ApiOperation({ summary: '新增报价配件' })
  @Perm(permissions.CREATE)
  async create(@Body() dto: SaleQuotationComponentDto): Promise<void> {
    await this.saleQuotationComponentService.create(dto);
  }

  @Put(':id')
  @ApiOperation({ summary: '更新报价配件' })
  @Perm(permissions.UPDATE)
  async update(@IdParam() id: number, @Body() dto: SaleQuotationComponentUpdateDto): Promise<void> {
    await this.saleQuotationComponentService.update(id, dto);
  }

  @Delete(':id')
  @ApiOperation({ summary: '删除报价配件' })
  @Perm(permissions.DELETE)
  async delete(@IdParam() id: number): Promise<void> {
    await this.saleQuotationComponentService.delete(id);
  }

  @Put('unlink-attachments/:id')
  @ApiOperation({ summary: '附件解除关联' })
  @Perm(permissions.UPDATE)
  async unlinkAttachments(
    @IdParam() id: number,
    @Body() { fileIds }: SaleQuotationComponentUpdateDto
  ): Promise<void> {
    await this.saleQuotationComponentService.unlinkAttachments(id, fileIds);
  }
}
