import { Injectable } from '@nestjs/common';
import { InjectEntityManager, InjectRepository } from '@nestjs/typeorm';
import { EntityManager, Not, Repository } from 'typeorm';
import { Pagination } from '~/helper/paginate/pagination';
import { paginate } from '~/helper/paginate';
import { BusinessException } from '~/common/exceptions/biz.exception';
import { ErrorEnum } from '~/constants/error-code.constant';
import { fieldSearch } from '~/shared/database/field-search';
import { SaleQuotationComponentEntity } from './sale_quotation_component.entity';
import { Storage } from '~/modules/tools/storage/storage.entity';
import {
  SaleQuotationComponentDto,
  SaleQuotationComponentQueryDto,
  SaleQuotationComponentUpdateDto
} from './sale_quotation_component.dto';

@Injectable()
export class SaleQuotationComponentService {
  constructor(
    @InjectEntityManager() private entityManager: EntityManager,
    @InjectRepository(SaleQuotationComponentEntity)
    private saleQuotationComponentRepository: Repository<SaleQuotationComponentEntity>,
    @InjectRepository(Storage)
    private storageRepository: Repository<Storage>
  ) {}

  buildSearchQuery() {
    return this.saleQuotationComponentRepository
      .createQueryBuilder('saleQuotationComponent')
      .leftJoin('saleQuotationComponent.unit', 'unit')
      .leftJoin('saleQuotationComponent.files', 'files')
      .addSelect(['files.id', 'files.path', 'unit.id', 'unit.label']);
  }

  /**
   * 分页查询所有
   */
  async findAll({
    page,
    pageSize,
    ...fields
  }: SaleQuotationComponentQueryDto): Promise<Pagination<SaleQuotationComponentEntity>> {
    const queryBuilder = this.buildSearchQuery()
      .where(fieldSearch(fields))
      .andWhere('saleQuotationComponent.isDelete = 0');

    return paginate<SaleQuotationComponentEntity>(queryBuilder, {
      page,
      pageSize
    });
  }

  /**
   * 新增
   */
  async create(dto: SaleQuotationComponentDto): Promise<void> {
    const { unitPrice, name, componentSpecification } = dto;
    const isExist = await this.saleQuotationComponentRepository.exist({
      where: {
        unitPrice,
        name,
        componentSpecification
      }
    });
    if (isExist) {
      throw new BusinessException(ErrorEnum.SALE_QUOTATION_COMPONENT_DUPLICATED);
    }
    await this.saleQuotationComponentRepository.insert(dto);
  }

  /**
   * 更新
   */
  async update(
    id: number,
    { fileIds, ...data }: Partial<SaleQuotationComponentUpdateDto>
  ): Promise<void> {
    await this.entityManager.transaction(async manager => {
      const beUpdateEntity = await manager.findOne(SaleQuotationComponentEntity, {
        where: {
          id
        }
      });
      const { unitPrice, name, componentSpecification } = beUpdateEntity;
      const isExist = await this.saleQuotationComponentRepository.exist({
        where: {
          id: Not(id),
          unitPrice,
          name,
          componentSpecification
        }
      });
      if (isExist) {
        throw new BusinessException(ErrorEnum.SALE_QUOTATION_COMPONENT_DUPLICATED);
      }
      await manager.update(SaleQuotationComponentEntity, id, {
        ...data
      });
      if (fileIds?.length) {
        const count = await this.storageRepository
          .createQueryBuilder('storage')
          .where('storage.id in(:fileIds)', { fileIds })
          .getCount();
        if (count !== fileIds?.length) {
          throw new BusinessException(ErrorEnum.STORAGE_NOT_FOUND);
        }
        // 附件要批量插入
        await manager
          .createQueryBuilder()
          .relation(SaleQuotationComponentEntity, 'files')
          .of(id)
          .add(fileIds);
      }
    });
  }

  /**
   * 删除
   */
  async delete(id: number): Promise<void> {
    // 比较重要，做逻辑删除
    await this.saleQuotationComponentRepository.update(id, { isDelete: 1 });
  }

  /**
   * 获取单个信息
   */
  async info(id: number) {
    const info = await this.buildSearchQuery()
      .where({
        id
      })
      .andWhere('saleQuotationComponent.isDelete = 0')
      .getOne();
    return info;
  }

  /**
   * 解除附件关联
   * @param id 实体ID
   * @param fileIds 附件ID
   */
  async unlinkAttachments(id: number, fileIds: number[]) {
    await this.entityManager.transaction(async manager => {
      const saleQuotationComponent = await this.saleQuotationComponentRepository
        .createQueryBuilder('saleQuotationComponent')
        .leftJoinAndSelect('saleQuotationComponent.files', 'files')
        .where('saleQuotationComponent.id = :id', { id })
        .getOne();
      const linkedFiles = saleQuotationComponent.files
        .map(item => item.id)
        .filter(item => !fileIds.includes(item));
      // 附件要批量更新
      await manager
        .createQueryBuilder()
        .relation(SaleQuotationComponentEntity, 'files')
        .of(id)
        .addAndRemove(linkedFiles, saleQuotationComponent.files);
    });
  }
}
